from gophish.models import (Campaign, CampaignSummary, CampaignSummaries,
                            CampaignResults)
from gophish.api import APIEndpoint


class API(APIEndpoint):
    def __init__(self, api, endpoint='api/campaigns/'):
        """ Creates a new instance of the campaigns API """

        super(API, self).__init__(api, endpoint=endpoint, cls=Campaign)

    def get(self, campaign_id=None):
        """ Gets the details for one or more campaigns by ID """

        return super(API, self).get(resource_id=campaign_id)

    def post(self, campaign):
        """ Creates a new campaign """

        return super(API, self).post(campaign)

    def puts(self, campaign):
        """ Edits an existing campaign """

        return super(API, self).put(campaign)

    def delete(self, campaign_id):
        """ Deletes an existing campaign """

        return super(API, self).delete(campaign_id)

    def complete(self, campaign_id):
        """ Complete an existing campaign (Stop processing events) """

        return super(API, self).get(
            resource_id=campaign_id, resource_action='complete')

    def summary(self, campaign_id=None):
        """ Returns the campaign summary """
        resource_cls = CampaignSummary
        single_resource = False

        if not campaign_id:
            resource_cls = CampaignSummaries
            single_resource = True

        return super(API, self).get(
            resource_id=campaign_id,
            resource_action='summary',
            resource_cls=resource_cls,
            single_resource=single_resource)

    def results(self, campaign_id):
        """ Returns just the results for a given campaign """
        return super(API, self).get(
            resource_id=campaign_id,
            resource_action='results',
            resource_cls=CampaignResults)



from gophish.models import Template
from gophish.api import APIEndpoint


class API(APIEndpoint):
    def __init__(self, api, endpoint='api/templates/'):
        super(API, self).__init__(api, endpoint=endpoint, cls=Template)

    def get(self, template_id=None):
        """ Gets one or more templates """

        return super(API, self).get(resource_id=template_id)

    def post(self, template):
        """ Creates a new template """

        return super(API, self).post(template)

    def put(self, template):
        """ Edits a template """

        return super(API, self).put(template)

    def delete(self, template_id):
        """ Deletes a template by ID """

        return super(API, self).delete(template_id)



from gophish.models import Webhook
from gophish.api import APIEndpoint


class API(APIEndpoint):
    def __init__(self, api, endpoint='api/webhooks/'):
        super(API, self).__init__(api, endpoint=endpoint, cls=Webhook)

    def get(self, webhook_id=None):
        """Gets one or more webhooks

        Keyword Arguments:
            webhook_id {int} --  The ID of the Webhook (optional, default: {None})
        """

        return super(API, self).get(resource_id=webhook_id)

    def post(self, webhook):
        """Creates a new webhook

        Arguments:
            webhook {gophish.models.Webhook} -- The webhook to create
        """

        return super(API, self).post(webhook)

    def put(self, webhook):
        """Edits a webhook

        Arguments:
            webhook {gophish.models.Webhook} -- The updated webhook details
        """

        return super(API, self).put(webhook)

    def delete(self, webhook_id):
        """Deletes a webhook by ID

        Arguments:
            webhook_id {int} -- The ID of the webhook to delete
        """
        return super(API, self).delete(webhook_id)

    def validate(self, webhook_id):
        """Sends a validation payload to the webhook specified by the given ID

        Arguments:
            webhook_id {int} -- The ID of the webhook to validate
        """
        return self.request("POST",
                            resource_id=webhook_id,
                            resource_action='validate',
                            resource_cls=Webhook,
                            single_resource=True)



from datetime import datetime
from dateutil.tz import tzlocal

import json as _json
import dateutil.parser


def parse_date(datestr):
    """ Parses an ISO 8601 formatted date from Gophish """
    return dateutil.parser.parse(datestr)


class Model(object):
    def __init__(self):
        self._valid_properties = {}

    @classmethod
    def _is_builtin(cls, obj):
        return isinstance(obj, (int, float, str, list, dict, bool))

    def as_dict(self):
        """ Returns a dict representation of the resource """
        result = {}
        for key in self._valid_properties:
            val = getattr(self, key)
            if isinstance(val, datetime):
                val = val.isoformat()
            # Parse custom classes
            elif val and not Model._is_builtin(val):
                val = val.as_dict()
            # Parse lists of objects
            elif isinstance(val, list):
                # We only want to call as_dict in the case where the item
                # isn't a builtin type.
                for i in range(len(val)):
                    if Model._is_builtin(val[i]):
                        continue
                    val[i] = val[i].as_dict()
            # If it's a boolean, add it regardless of the value
            elif isinstance(val, bool):
                result[key] = val

            # Add it if it's not None
            if val:
                result[key] = val
        return result

    @classmethod
    def parse(cls, json):
        """Parse a JSON object into a model instance."""
        raise NotImplementedError


class Campaign(Model):
    _valid_properties = {
        'id': None,
        'name': None,
        'created_date': datetime.now(tzlocal()),
        'launch_date': datetime.now(tzlocal()),
        'send_by_date': None,
        'completed_date': None,
        'template': None,
        'page': None,
        'results': [],
        'status': None,
        'timeline': [],
        'smtp': None,
        'url': None,
        'groups': [],
    }

    def __init__(self, **kwargs):
        """ Creates a new campaign instance """
        for key, default in Campaign._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        campaign = cls()
        for key, val in json.items():
            # TODO Add date parsing
            if key == 'results':
                results = [Result.parse(result) for result in val]
                setattr(campaign, key, results)
            elif key == 'timeline':
                if val is not None:
                    timeline = [TimelineEntry.parse(entry) for entry in val]
                    setattr(campaign, key, timeline)
            elif key == 'template':
                setattr(campaign, key, Template.parse(val))
            elif key == 'page':
                setattr(campaign, key, Page.parse(val))
            elif key == 'smtp':
                setattr(campaign, key, SMTP.parse(val))
            elif key in cls._valid_properties:
                setattr(campaign, key, val)
        return campaign


class CampaignSummaries(Model):
    ''' Represents a list of campaign summary objects '''
    _valid_properties = {'total': None, 'campaigns': None}

    def __init__(self):
        """ Creates a new instance of the campaign summaries"""
        for key, default in CampaignSummaries._valid_properties.items():
            setattr(self, key, default)

    @classmethod
    def parse(cls, json):
        campaign_summaries = cls()
        for key, val in json.items():
            # TODO Add date parsing
            if key == 'campaigns':
                summaries = [CampaignSummary.parse(summary) for summary in val]
                setattr(campaign_summaries, key, summaries)
            elif key in cls._valid_properties:
                setattr(campaign_summaries, key, val)
        return campaign_summaries


class CampaignSummary(Model):
    ''' Represents a campaign summary object '''
    _valid_properties = {
        'id': None,
        'name': None,
        'status': None,
        'created_date': None,
        'send_by_date': None,
        'launch_date': None,
        'completed_date': None,
        'stats': None
    }

    def __init__(self):
        for key, default in CampaignSummary._valid_properties.items():
            setattr(self, key, default)

    @classmethod
    def parse(cls, json):
        summary = cls()
        for key, val in json.items():
            # TODO Add date parsing
            if key == 'stats':
                stats = Stat.parse(val)
                setattr(summary, key, stats)
            elif key in cls._valid_properties:
                setattr(summary, key, val)
        return summary


class Stat(Model):
    _valid_properties = {
        'total': None,
        'sent': None,
        'opened': None,
        'clicked': None,
        'submitted_data': None,
        'error': None
    }

    def __init__(self):
        for key, default in Stat._valid_properties.items():
            setattr(self, key, default)

    @classmethod
    def parse(cls, json):
        stat = cls()
        for key, val in json.items():
            if key in cls._valid_properties:
                setattr(stat, key, val)
        return stat


class CampaignResults(Model):
    ''' Represents a succinct view of campaign results '''
    _valid_properties = {
        'id': None,
        'name': None,
        'results': [],
        'status': None,
        'timeline': [],
    }

    def __init__(self, **kwargs):
        """ Creates a new instance of the campaign results object"""
        for key, default in CampaignResults._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        campaign_results = cls()
        for key, val in json.items():
            # TODO Add date parsing
            if key == 'results':
                results = [Result.parse(result) for result in val]
                setattr(campaign_results, key, results)
            elif key == 'timeline':
                if val is not None:
                    timeline = [TimelineEntry.parse(entry) for entry in val]
                    setattr(campaign_results, key, timeline)
            elif key in cls._valid_properties:
                setattr(campaign_results, key, val)
        return campaign_results


class Result(Model):
    _valid_properties = {
        'id': None,
        'first_name': None,
        'last_name': None,
        'email': None,
        'position': None,
        'ip': None,
        'latitude': None,
        'longitude': None,
        'status': None
    }

    def __init__(self, **kwargs):
        for key, default in Result._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        result = cls()
        for key, val in json.items():
            if key in cls._valid_properties:
                setattr(result, key, val)
        return result


class TimelineEntry(Model):
    _valid_properties = {
        'email': None,
        'time': None,
        'message': None,
        'details': None
    }

    def __init__(self):
        ''' Creates a new instance of a timeline entry '''
        for key, default in TimelineEntry._valid_properties.items():
            setattr(self, key, default)

    @classmethod
    def parse(cls, json):
        entry = cls()
        for key, val in json.items():
            if key == 'details' and val != "":
                details = _json.loads(val)
                setattr(entry, key, details)
            elif key in cls._valid_properties:
                setattr(entry, key, val)
        return entry


class User(Model):
    """ User contains the attributes for a member of a group
        used in Gophish """
    _valid_properties = {
        'id': None,
        'first_name': None,
        'last_name': None,
        'email': None,
        'position': None
    }

    def __init__(self, **kwargs):
        for key, default in User._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        user = cls()
        for key, val in json.items():
            if key in cls._valid_properties:
                setattr(user, key, val)
        return user


class Group(Model):
    """ Groups contain one or more users """
    _valid_properties = {
        'id': None,
        'name': None,
        'modified_date': datetime.now(tzlocal()),
        'targets': []
    }

    def __init__(self, **kwargs):
        for key, default in Group._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        group = cls()
        for key, val in json.items():
            if key == 'targets':
                targets = [User.parse(user) for user in val]
                setattr(group, key, targets)
            elif key == 'modified_date':
                setattr(group, key, parse_date(val))
            elif key in cls._valid_properties:
                setattr(group, key, val)
        return group


class SMTP(Model):
    _valid_properties = {
        'id': None,
        'interface_type': 'SMTP',
        'name': None,
        'host': None,
        'username': None,
        'password': None,
        'from_address': None,
        'ignore_cert_errors': False,
        'modified_date': datetime.now(tzlocal()),
        'headers': []
    }

    def __init__(self, **kwargs):
        for key, default in SMTP._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        smtp = cls()
        for key, val in json.items():
            if key == 'modified_date':
                setattr(smtp, key, parse_date(val))
            elif key in cls._valid_properties:
                setattr(smtp, key, val)
        return smtp


class Template(Model):
    _valid_properties = {
        'id': None,
        'name': None,
        'text': None,
        'html': None,
        'modified_date': datetime.now(tzlocal()),
        'subject': None,
        'attachments': []
    }

    def __init__(self, **kwargs):
        for key, default in Template._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        template = cls()
        for key, val in json.items():
            if key == 'modified_date':
                setattr(template, key, parse_date(val))
            elif key == 'attachments' and val:
                attachments = [
                    Attachment.parse(attachment) for attachment in val
                ]
                setattr(template, key, attachments)
            elif key in cls._valid_properties:
                setattr(template, key, val)
        return template


class Page(Model):
    _valid_properties = {
        'id': None,
        'name': None,
        'html': None,
        'modified_date': datetime.now(tzlocal()),
        'capture_credentials': False,
        'capture_passwords': False,
        'redirect_url': None
    }

    def __init__(self, **kwargs):
        for key, default in Page._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        page = cls()
        for key, val in json.items():
            if key == 'modified_date':
                setattr(page, key, parse_date(val))
            elif key in cls._valid_properties:
                setattr(page, key, val)
        return page


class Attachment(Model):
    _valid_properties = {'content': None, 'type': None, 'name': None}

    @classmethod
    def parse(cls, json):
        attachment = cls()
        for key, val in json.items():
            if key in cls._valid_properties:
                setattr(attachment, key, val)
        return attachment


class Webhook(Model):
    _valid_properties = {
        'id': None,
        'name': None,
        'url': None,
        'secret': None,
        'is_active': None
    }

    def __init__(self, **kwargs):
        for key, default in Webhook._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        webhook = cls()
        for key, val in json.items():
            if key in cls._valid_properties:
                setattr(webhook, key, val)
        return webhook


class IMAP(Model):
    _valid_properties = {
        'enabled': None,
        'host': None,
        'port': None,
        'username': None,
        'password': None,
        'tls': None,
        'folder': None,
        'restrict_domain': None,
        'delete_reported_campaign_email': None,
        'last_login': None,
        'modified_date': None,
        'imap_freq': None
    }

    def __init__(self, **kwargs):
        for key, default in IMAP._valid_properties.items():
            setattr(self, key, kwargs.get(key, default))

    @classmethod
    def parse(cls, json):
        imap = cls()
        for key, val in json.items():
            if key in cls._valid_properties:
                setattr(imap, key, val)
        return imap


class Success(Exception, Model):
    _valid_properties = {'message': None, 'success': None, 'data': None}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __str__(self):
        return self.message

    @classmethod
    def parse(cls, json):
        success = cls()
        for key, val in json.items():
            if key in cls._valid_properties:
                setattr(success, key, val)
        return success


class Error(Exception, Model):
    _valid_properties = {'message': None, 'success': None, 'data': None}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __str__(self):
        return self.message

    def __repr__(self):
        return _json.dumps(self.as_dict())

    @classmethod
    def parse(cls, json):
        error = cls()
        for key, val in json.items():
            if key in cls._valid_properties:
                setattr(error, key, val)
        return error



from gophish.models import Error
'''
api.py

Base API endpoint class that abstracts basic CRUD operations.
'''


class APIEndpoint(object):
    """
    Represents an API endpoint for Gophish, containing common patterns
    for CRUD operations.
    """
    def __init__(self, api, endpoint=None, cls=None):
        """ Creates an instance of the APIEndpoint class.

        Args:
            api - Gophish.client - The authenticated REST client
            endpoint - str - The URL path to the resource endpoint
            cls - gophish.models.Model - The Class to use when parsing results
        """
        self.api = api
        self.endpoint = endpoint
        self._cls = cls

    def _build_url(self, *parts):
        """Builds a path to an API resource by joining the individual parts
        with a slash (/).

        This is used instead of urljoin since we're given relative URL parts
        which need to be chained together.

        Returns:
            str -- The parts joined with a slash
        """

        return '/'.join(str(part).rstrip('/') for part in parts)

    def request(self,
                method,
                body=None,
                resource_id=None,
                resource_action=None,
                resource_cls=None,
                single_resource=False):

        endpoint = self.endpoint

        if not resource_cls:
            resource_cls = self._cls

        if resource_id:
            endpoint = self._build_url(endpoint, resource_id)

        if resource_action:
            endpoint = self._build_url(endpoint, resource_action)

        response = self.api.execute(method, endpoint, json=body)
        if not response.ok:
            raise Error.parse(response.json())

        if resource_id or single_resource:
            return resource_cls.parse(response.json())

        return [resource_cls.parse(resource) for resource in response.json()]

    def get(self,
            resource_id=None,
            resource_action=None,
            resource_cls=None,
            single_resource=False):
        """ Gets the details for one or more resources by ID

        Args:
            cls - gophish.models.Model - The resource class
            resource_id - str - The endpoint (URL path) for the resource
            resource_action - str - An action to perform on the resource
            resource_cls - cls - A class to use for parsing, if different than
                the base resource
            single_resource - bool - An override to tell Gophish that even
                though we aren't requesting a single resource, we expect a
                single response object

        Returns:
            One or more instances of cls parsed from the returned JSON
        """

        return self.request("GET",
                            resource_id=resource_id,
                            resource_action=resource_action,
                            resource_cls=resource_cls,
                            single_resource=single_resource)

    def post(self, resource):
        """ Creates a new instance of the resource.

        Args:
            resource - gophish.models.Model - The resource instance

        """
        response = self.api.execute("POST",
                                    self.endpoint,
                                    json=(resource.as_dict()))

        if not response.ok:
            raise Error.parse(response.json())

        return self._cls.parse(response.json())

    def put(self, resource):
        """ Edits an existing resource

        Args:
            resource - gophish.models.Model - The resource instance
        """

        endpoint = self.endpoint

        if resource.id:
            endpoint = self._build_url(endpoint, resource.id)

        response = self.api.execute("PUT", endpoint, json=resource.as_dict())

        if not response.ok:
            raise Error.parse(response.json())

        return self._cls.parse(response.json())

    def delete(self, resource_id):
        """ Deletes an existing resource

        Args:
            resource_id - int - The resource ID to be deleted
        """

        endpoint = '{}/{}'.format(self.endpoint, resource_id)

        response = self.api.execute("DELETE", endpoint)

        if not response.ok:
            raise Error.parse(response.json())

        return self._cls.parse(response.json())



from gophish.models import SMTP
from gophish.api import APIEndpoint


class API(APIEndpoint):
    def __init__(self, api, endpoint='api/smtp/'):
        super(API, self).__init__(api, endpoint=endpoint, cls=SMTP)

    def get(self, smtp_id=None):
        """ Gets one or more SMTP sending profiles """

        return super(API, self).get(resource_id=smtp_id)

    def post(self, smtp):
        """ Creates a new SMTP sending profile """

        return super(API, self).post(smtp)

    def put(self, smtp):
        """ Edits an SMTP sending profile """

        return super(API, self).put(smtp)

    def delete(self, smtp_id):
        """ Deletes an SMTP sending profile by ID """

        return super(API, self).delete(smtp_id)



from gophish.models import IMAP, Success
from gophish.api import APIEndpoint


class API(APIEndpoint):
    def __init__(self, api, endpoint='api/imap/'):
        super(API, self).__init__(api, endpoint=endpoint, cls=IMAP)

    def get(self):
        """Gets the configured IMAP settings
        """

        return super(API, self).get()

    def post(self, imap):
        """Updates the IMAP settings

        Arguments:
            imap {gophish.models.IMAP} -- The IMAP settings to configure
        """

        return super(API, self).post(imap)

    def validate(self, imap):
        """Sends a validation payload to the webhook specified by the given ID

        Arguments:
            webhook_id {int} -- The ID of the webhook to validate
        """
        return self.request("POST",
                            body=imap.as_dict(),
                            resource_action='validate',
                            resource_cls=Success,
                            single_resource=True)



from gophish.models import Page
from gophish.api import APIEndpoint


class API(APIEndpoint):
    def __init__(self, api, endpoint='api/pages/'):
        super(API, self).__init__(api, endpoint=endpoint, cls=Page)

    def get(self, page_id=None):
        """ Gets one or more pages """

        return super(API, self).get(resource_id=page_id)

    def post(self, page):
        """ Creates a new page """

        return super(API, self).post(page)

    def put(self, page):
        """ Edits a page """

        return super(API, self).put(page)

    def delete(self, page_id):
        """ Deletes a page by ID """

        return super(API, self).delete(page_id)



"""This is the setup module for the Python Gophish API client."""
from setuptools import setup

setup(
    name="gophish",
    packages=["gophish", "gophish.api"],
    version="0.4.0",
    description="Python API Client for Gophish",
    author="Jordan Wright",
    author_email="python@getgophish.com",
    url="https://github.com/gophish/api-client-python",
    license="MIT",
    download_url="https://github.com/gophish/api-client-python/tarball/0.4.0",
    keywords=["gophish"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
    ],
    install_requires=[
        "appdirs==1.4.0",
        "packaging==16.8",
        "pyparsing==2.1.10",
        "python-dateutil==2.6.0",
        "requests>=2.20.0",
        "six==1.10.0",
    ],
)



from gophish.models import Group
from gophish.api import APIEndpoint


class API(APIEndpoint):
    def __init__(self, api, endpoint='api/groups/'):
        super(API, self).__init__(api, endpoint=endpoint, cls=Group)

    def get(self, group_id=None):
        """ Gets one or more groups """
        return super(API, self).get(resource_id=group_id)

    def post(self, group):
        """ Creates a new group """
        return super(API, self).post(group)

    def put(self, group):
        """ Edits a group """
        return super(API, self).put(group)

    def delete(self, group_id):
        """ Deletes a group by ID """
        return super(API, self).delete(group_id)



import requests

from gophish.api import (campaigns, groups, imap, pages, smtp, templates,
                         webhooks)

DEFAULT_URL = 'http://localhost:3333'


class GophishClient(object):
    """ A standard HTTP REST client used by Gophish """
    def __init__(self, api_key, host=DEFAULT_URL, **kwargs):
        self.api_key = api_key
        if host.endswith('/'):
            self.host = host
        else:
            self.host = host + '/'
        self._client_kwargs = kwargs

    def execute(self, method, path, **kwargs):
        """ Executes a request to a given endpoint, returning the result """

        url = "{}{}".format(self.host, path)
        kwargs.update(self._client_kwargs)
        response = requests.request(
            method,
            url,
            headers={"Authorization": "Bearer {}".format(self.api_key)},
            **kwargs)
        return response


class Gophish(object):
    def __init__(self,
                 api_key,
                 host=DEFAULT_URL,
                 client=GophishClient,
                 **kwargs):
        self.client = client(api_key, host=host, **kwargs)
        self.campaigns = campaigns.API(self.client)
        self.groups = groups.API(self.client)
        self.imap = imap.API(self.client)
        self.pages = pages.API(self.client)
        self.smtp = smtp.API(self.client)
        self.templates = templates.API(self.client)
        self.webhooks = webhooks.API(self.client)
